package sound.audioDigitizer;

/**
 * Created by IntelliJ IDEA.
 * User: Douglas Lyon
 * Date: Dec 14, 2004
 * Time: 6:28:03 AM
 * Copyright DocJava, Inc.
 */

import sound.Utils;

import javax.sound.sampled.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.ByteArrayOutputStream;

public class CapturePlaybackFrame extends JFrame {
    boolean stopCapture = false;
    ByteArrayOutputStream byteArrayOutputStream;

    public static void main(String args[]) {
        new CapturePlaybackFrame();
    }//end main

    public CapturePlaybackFrame() {
        final JButton captureBtn = new JButton("Capture");
        final JButton stopBtn = new JButton("Stop");
        final JButton playBtn = new JButton("Playback");
        captureBtn.setEnabled(true);
        stopBtn.setEnabled(false);
        playBtn.setEnabled(false);
//Register anonymous listeners
        captureBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                captureBtn.setEnabled(false);
                stopBtn.setEnabled(true);

                playBtn.setEnabled(false);
//Capture input data from the microphone until the Stop button is clicked.
                captureAudio();
            }//end actionPerformed
        }//end ActionListener
        ); //end addActionListener()
        getContentPane().add(captureBtn);
        stopBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                captureBtn.setEnabled(true);
                stopBtn.setEnabled(false);
                playBtn.setEnabled(true);
//Terminate the capturing of input data from the microphone.
                stopCapture = true;
            }//end actionPerformed
        }//end ActionListener
        );//end addActionListener()
        getContentPane().add(stopBtn);
        playBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
//Play back all of the data that was saved during capture.
                PlayThread.playAudio(byteArrayOutputStream);
            }//end actionPerformed
        }//end ActionListener
        );//end addActionListener()
        getContentPane().add(playBtn);
        getContentPane().setLayout(new FlowLayout());
        setTitle("Capture/Playback Demo");
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setSize(250, 70);
        setVisible(true);
    }
    private void captureAudio() {

        try {
            Thread captureThread = new Thread(new CaptureThread());
            captureThread.start();
        } catch (Exception e) {
            System.out.println(e);
            System.exit(0);
        }//end catch
    }//end captureAudio method
//This method plays back the audio data that has been saved in the
// ByteArrayOutputStream

//This method creates and returns an
// AudioFormat object for a given set
// of format parameters. If these
// parameters dont work well for
// you, try some of the other
// allowable parameter values, which
// are shown in comments following
// the declarations.
//===================================//
//Inner class to capture data from
// microphone
    class CaptureThread extends Thread {
//An arbitrary-size temporary holding buffer
        byte tempBuffer[] = new byte[10000];
        AudioFormat audioFormat = Utils.get8khzMono8Format();
        DataLine.Info dataLineInfo = new DataLine.Info(TargetDataLine.class,
                audioFormat);

        TargetDataLine targetDataLine;

        public CaptureThread() {
            try {
                targetDataLine  = (TargetDataLine) AudioSystem.getLine(dataLineInfo);
                targetDataLine.open(audioFormat);
                targetDataLine.start();
            } catch (LineUnavailableException e) {
                e.printStackTrace();
            }
        }

        public void run() {
            byteArrayOutputStream = new ByteArrayOutputStream();

            stopCapture = false;
            try { //Loop until stopCapture is set by another thread that
// services the Stop button.
                while (!stopCapture) {
//Read data from the internal
// buffer of the data line.
                    int cnt = targetDataLine.read(tempBuffer, 0, tempBuffer.length);
                    if (cnt > 0) {
//Save data in output stream object.
                        byteArrayOutputStream.write(tempBuffer, 0, cnt);
                    }//end if
                }//end while
                byteArrayOutputStream.close();
            } catch (Exception e) {
                System.out.println(e);
                System.exit(0);
            }//end catch
        }//end run
    }
}
